class WebAppVersion : IComparable {
    [int]$Major
    [int]$Minor

    WebAppVersion() {
    }

    WebAppVersion([int] $major, [int] $minor) {
        $this.Major = $major
        $this.Minor = $minor
    }
    
    [bool] Equals([Object] $that) {
      return $this.Major -eq $that.Major -and $this.Minor -eq $that.Minor
    }

    [int] CompareTo([Object] $that) {
        if ($this.Major -ne $that.Major) {
            return $this.Major - $that.Major
        } else {
            return $this.Minor - $that.Minor
        }
    }

   [int] GetHashCode() {
      return ("{0}{1}" -f $this.Major, $this.Minor).GetHashCode()
   }

   [string] ToString() {
       return "{0}.{1}" -f $this.Major, $this.Minor
   }
}

class WebAppInstallationInfo {
    [Object] $Site
    [WebAppVersion] $Version
    [string] $BinPath

    WebAppInstallationInfo([Object] $site, [WebAppVersion]$version, [string]$binPath) {
        $this.Site = $site
        $this.Version = $version
        $this.BinPath = $binPath
    }
}

class ValidationResult {
    [int]$Successes
    [int]$Warnings
    [int]$Errors
    
    static [ValidationResult] op_Addition([ValidationResult]$first, [ValidationResult]$second) {
        $result =  [ValidationResult]::new()
        $result.Successes = $first.Successes + $second.Successes
        $result.Warnings = $first.Warnings + $second.Warnings
        $result.Errors = $first.Errors + $second.Errors
        return $result
    }
}

function Set-ClippedValidationResult{
    param(
        [Parameter(Mandatory = $true)]
        [ValidationResult] $ValidationResult
    )

    if ($ValidationResult.Errors -gt 0) {
        $ValidationResult.Errors = 1
        $ValidationResult.Warnings = 0
        $ValidationResult.Successes = 0
    }

    if ($ValidationResult.Warnings -gt 0) {
        $ValidationResult.Warnings = 1
        $ValidationResult.Successes = 0    }

    if ($ValidationResult.Successes -gt 0) {
        $ValidationResult.Successes = 1
    }
}

function Set-SuccessesEmptyValidationResult{
    param(
        [Parameter(Mandatory = $true)]
        [ValidationResult] $ValidationResult
    )

    if ($ValidationResult.Warnings -eq 0 -and $ValidationResult.Successes -eq 0 -and $ValidationResult.Errors -eq 0) {
        # Consider empty validation result as successful
        $ValidationResult.Successes++
    }
}

function Write-ValidationResult {
    param(
        [Parameter(Mandatory = $true)]
        [ValidationResult] $ValidationResult,
        [Parameter(Mandatory = $false)]
        [string] $ValidationSubject,
        [Parameter(Mandatory = $false)]
        [bool] $OutputVerbose = $false
    )

    $message = "{0} validations: {1} succeeded, {2} failed and {3} warning(s)." `
        -f $ValidationSubject, $ValidationResult.Successes, $ValidationResult.Errors, $ValidationResult.Warnings
 
    if ($OutputVerbose) {
        Write-Verbose $message
    } else {
        Write-MessageWithValidation -Message $message -ValidationResult $ValidationResult
    }
}

function Write-MessageWithValidation {
    param(
        [Parameter(Mandatory = $true)]
        [string] $Message,
        [Parameter(Mandatory = $true)]
        [ValidationResult] $ValidationResult
    )
    
    if($ValidationResult.Errors -gt 0){
        Write-ErrorMessage $Message
    }
    elseif ($ValidationResult.Warnings -gt 0){
        Write-WarningMessage $Message
    }
    else{
        Write-SuccessMessage $Message
    }
}